class Design
  include Mongoid::Document
  include Mongoid::Timestamps

  field :title
  field :author
  field :intro
  field :version

  has_many :pages
  
  embeds_one :layout
  embeds_one :default_css, :class_name => "Stylesheet" 
  embeds_one :reset_css, :class_name => "Stylesheet"
  embeds_many :stylesheets
  embeds_many :themes
  embeds_many :javascripts
  embeds_many :images
  embeds_many :custom_images, as: :design_image
  
  validates_presence_of :title
  validates_presence_of :author
  
  after_save :procs_embedded_objects
  # after_destroy :del_embedded_objects
 
  def javascripts=(*attrs)
    self.files = (attrs << 'javascripts')
  end
  
  def themes=(*attrs)
    self.files = (attrs << 'themes')
  end
  
  def images=(*attrs)
    self.files = (attrs << 'images')
  end
  
  # Update or create the attribute_model records
  def files=(attrs)
    case attrs.last
      when 'layout'
        files = self.layout.build
      else
        files = eval(attrs.last)
    end
    attrs[0].each do |a|
      if a[:id].blank? && !a[:file].blank?
        files.build(:file => a[:file], :to_save => true)
      else
        files.each do |file|
          if file.id.to_s == a[:id]
             file.to_destroy = a[:to_destroy]
          end
        end
      end
    end
  end
  
  protected
  
  def del_embedded_objects
    [[self.layout],self.themes, self.javascripts, self.images].each do |objects|
      objects.each do |object|
        object.destroy
      end
     end
  end
  
  def procs_embedded_objects
    [self.javascripts, self.images].each do |objects|
      objects.each do |object|
        process_object(object)
      end
    end
    if self.layout.to_save
      self.layout.to_save = false
      self.layout.save
    end
    [[self.default_css], [self.reset_css], self.themes].each do |objects|
      objects.each do |object|
        if object
          to_parse_url = object.to_save ? true : false
          process_object(object)
          object.parse_urls if to_parse_url
        end
      end
    end
  end
  
  def process_object(object)
    if object.to_save
      if object.file_filename.nil?
        new_object = self.send(object._type.downcase.pluralize).build(object)
        object.destroy
        new_object.to_save = false
        new_object.save
      else
        object.to_save = false
        object.save
      end
    end
    if object.to_destroy
      object.destroy
    end
  end
  
end
