class User
  
  include Mongoid::Document
  include Mongoid::Timestamps
 
  devise :database_authenticatable, :recoverable, :rememberable, :trackable, :registerable
  
  mount_uploader :avatar, AvatarUploader
  
  field :admin, :type => Boolean, :default => true
  field :active_role
  
  has_many :user_attributes
  has_many :user_roles
  has_many :user_infos
  
  # Update or create the user_role records
  def user_roles=(*attrs)
    attrs[0].each do |roles|
      if roles[:id].blank?
        user_roles.build(roles)
      else
        user_role = user_roles.detect {|a| a.id.to_s == roles[:id].to_s }
        user_role.update_attributes(roles)
      end
    end
  end
  
  # Update or create the user_info records
  def user_infos=(*attrs)
    attrs[0].each do |infos|
      if infos[:id].blank?
        user_infos.build(infos)
      else
        user_info = user_infos.detect {|a| a.id.to_s == infos[:id].to_s }
        user_info.update_attributes(infos)
      end
    end
  end
  
  # Get an user_info from model key
  def get_info_from_model_key(key)
    self.user_infos.detect {|a| a.key.to_s == key.to_s }
  end
  
  # Get an user_role from model key
  def get_role_from_model_key(key)
    self.user_roles.detect {|a| a.key.to_s == key.to_s }
  end
  
  def name
    infos = self.user_infos.detect {|info| info.key.to_s.eql?('profile') }
    if infos
      first_name = infos.attribute_values.detect {|value| value.key.to_s.eql?('first_name') }[I18n.locale.to_s] rescue nil
      last_name = infos.attribute_values.detect {|value| value.key.to_s.eql?('last_name') }[I18n.locale.to_s] rescue nil
      return "#{first_name} #{last_name}"
    else
      return nil
    end
  end
  
end
